.details-required {
  position: relative;
}

.details-required div {
  position: absolute;
  bottom: 0;
  right: 0;
  opacity: 0;
  visibility: hidden;
  -webkit-transition:
    visibility 0s,
    opacity 0.5s linear;
  transition:
    visibility 0s,
    opacity 0.5s linear;
}

.details-required:hover {
  cursor: tooltip;
}

.details-required:hover div {
  padding: 1rem;
  padding-bottom: 0rem;
  margin-right: 8rem;
  visibility: visible;
  border-radius: 1rem;
  background-color: rgba(242, 242, 242, 0.892);
  opacity: 1;
}

.details-required:hover div .details-desc {
  right: -40%;
}

.hover-details {
  display: flex;
  flex-direction: row;
  justify-content: space-between;
  gap: 0.7rem;
  background-color: var(--bs-body-bg) !important;
  color: var(--bs-body-color);
  border: 1px solid var(--bs-primary);
  text-wrap: nowrap;
}

.table-implementation-name:hover {
  cursor: pointer;
}
